<?php
class DashWidgetManager {
	var $absPath, $urlPath, $nonce;
	function __construct($mainFile) {
		$this->pluginDirPath = $this->get_plugin_dir();
		$this->absPath = $this->pluginDirPath . "/" . plugin_basename(dirname($mainFile));
		$this->urlPath = plugins_url("", $mainFile);
		$this->add_hooks($mainFile);
	}

	function get_plugin_dir() {
		return preg_replace('/\\' . DIRECTORY_SEPARATOR . 'dash-widget-manager\\' . DIRECTORY_SEPARATOR . 'lib$/', '', dirname(__FILE__));
	}

	function add_hooks($mainFile) {
		register_activation_hook($mainFile,array($this, 'activate'));
		add_action('wp_dashboard_setup', array($this, 'manage_dash_setup'), 100);
		add_filter('wp_dashboard_widgets', array($this, 'manage_widgets'), 100, 1);
		add_action('init', array($this, 'init'));

		add_action('admin_head', array($this, 'dashboard_head'));

		if (get_option('dwm_disable_core_update_message') == '1') {
			add_filter( 'pre_site_transient_update_core', array($this, 'hide_core_update'));
		}
		if (get_option('dwm_disable_plugin_update_message') == '1') {
			add_filter( 'admin_head', array($this, 'hide_plugin_update'));
		}

		add_action('wp_ajax_dwm_save_widget_options', array($this, 'save_widget_options'));
	}
	
	function dashboard_head() {
		$screen = get_current_screen();
		if ($screen->base == 'dashboard') {
			require_once($this->absPath . "/tpl/dashboardHead.php");
		}
	}
	
	function activate() {
		global $wpdb;
		if (get_option($wpdb->prefix.'dash_widget_manager_core_update_message') == 0) {
			update_option('dwm_disable_core_update_message', '1');
		}
		if (get_option($wpdb->prefix.'dash_widget_manager_plugin_update_message') == 0) {
			update_option('dwm_disable_plugin_update_message', '1');
		}
		if (is_array(get_option($wpdb->prefix.'dash_widget_manager_good_widgets')) && is_array(get_option($wpdb->prefix.'dash_widget_manager_registered_widgets'))) {
			$editableRoles = get_editable_roles();
			$availableRoles = array();
			foreach($editableRoles as $roleID=>$role) {
				if ($roleID != 'administrator') {
					$availableRoles[] = $roleID;
				}
			}
			
			$goodWidgets = get_option($wpdb->prefix.'dash_widget_manager_good_widgets');
			$disabledWidgets = array();
			foreach(get_option($wpdb->prefix.'dash_widget_manager_registered_widgets') as $columnKey=>$columnGroup) {
				foreach($columnGroup as $columnTypeKey=>$widgetColumn) {
					if (sizeof($widgetColumn) > 0) {
						foreach ($widgetColumn as $widgetID=>$widget) {
							if (!in_array($widgetID, $goodWidgets)) {
								$disabledWidgets[$widgetID] = $availableRoles;
							}
						}
					}
				}
			}
			update_option('dwm_disabled_widgets', $disabledWidgets);
		}

		delete_option($wpdb->prefix.'dash_widget_manager_core_update_message');
		delete_option($wpdb->prefix.'dash_widget_manager_plugin_update_message');
		delete_option($wpdb->prefix.'dash_widget_manager_good_widgets');
		delete_option($wpdb->prefix.'dash_widget_manager_registered_widgets');
		
	}
	
	function init() {
		##Create nonce
		$this->nonce = wp_create_nonce(plugin_basename(__FILE__));
	}
	
	function verify_nonce($nonce) {
		return wp_verify_nonce(sanitize_text_field($nonce), plugin_basename(__FILE__) );
	}
	
	function save_widget_options() {
		if (!$this->verify_nonce($_POST['dwm_nonce'])) {
			print "You dont have permissions to access this page.";
			die();
		}
		
		if (isset($_POST['dwm_disable_core_update_message']) && $_POST['dwm_disable_core_update_message'] == 'true') {
			update_option('dwm_disable_core_update_message', '1');
		} else {
			update_option('dwm_disable_core_update_message', '0');
		}
		
		if (isset($_POST['dwm_disable_plugin_update_message']) && $_POST['dwm_disable_plugin_update_message'] == 'true') {
			update_option('dwm_disable_plugin_update_message', '1');
		} else {
			update_option('dwm_disable_plugin_update_message', '0');
		}
		
		$postedWidgets = $_POST['disabled_widgets'];
		$disabledWidgets = array();
		foreach($postedWidgets as $widgetKey=>$widgetRoles) {
			$disabledWidgets[$widgetKey] = array();
			foreach($widgetRoles as $roleID) {
				$disabledWidgets[$widgetKey][] = sanitize_text_field($roleID);
			}
		}
		update_option('dwm_disabled_widgets', $disabledWidgets);
		die();
	}
	
	function dashboard_widget() {
		global $wp_meta_boxes;
		$editableRoles = get_editable_roles();
		$disabledWidgets = get_option('dwm_disabled_widgets');
		
		$disableCoreUpdateMessage = get_option('dwm_disable_core_update_message', '0');
		$disablePluginUpdateMessage = get_option('dwm_disable_plugin_update_message', '0');
		include($this->absPath.'/tpl/dashboardWidget.php');
	}
	
	function manage_dash_setup() {
		global $wp_meta_boxes;
		
		$disabledWidgets = get_option('dwm_disabled_widgets');
		$user=wp_get_current_user();
		if (is_array($wp_meta_boxes['dashboard'])) {
			if (!current_user_can('administrator')) {
				if (is_array($disabledWidgets)) {
					foreach($wp_meta_boxes['dashboard'] as $columnKey=>$columnGroup) {
						foreach($columnGroup as $columnTypeKey=>$widgetColumn) {
							if (sizeof($widgetColumn) > 0) {
								foreach ($widgetColumn as $widgetID=>$widget) {
									if (array_key_exists($widgetID, $disabledWidgets)) {
										foreach($user->roles as $role) {
											if (in_array($role, $disabledWidgets[$widgetID])) {
												remove_meta_box($widgetID, 'dashboard', $columnKey);
												unset($wp_meta_boxes['dashboard'][$columnKey][$columnTypeKey][$widgetID]);
												break;
											}
										}
									}
								}
							}
						}
					}
				}
			} else {
				wp_add_dashboard_widget('dwm_dashboard_widget', __('Manage Dashboard Widgets'), array($this, 'dashboard_widget'));
			}
		}
		
	}

	function manage_widgets($widgets) {
		if (!current_user_can('administrator')) {
			$disabledWidgets = get_option('dwm_disabled_widgets');
			for($i=0; $i < sizeof($widgets); $i++) {
				if (array_key_exists($widgets[$i], $disabledWidgets)) {
					unset($widgets[$i]);
				}
			}
		}
		return $widgets;
	}

	function hide_plugin_update() {
		if (!current_user_can('administrator')) {
			?>
			<style type="text/css">
				.plugin-update {
					display: none;
				}
			</style>
			<?php
		}
	}

	function hide_core_update($message) {
		if (current_user_can('administrator')) {
			return $message;
		} else {
			return null;
		}
	}

}
?>